#include <stdlib.h>
#include "myLib.h"

u16 *videoBuffer = (u16 *) 0x6000000;

void setPixel(int row, int col, u16 color) {
    videoBuffer[OFFSET(row, col, SCREENWIDTH)] = color;
}

void drawRect(int row, int col, int height, int width, u16 color) {

    int rowfinal, colfinal, i, j;
    rowfinal = row + height;
    colfinal = col + width;

    for (i = row; i <= rowfinal; i++) {
        for (j = col; j <= colfinal; j++) {
            setPixel(i, j, color);
        }
    }
}

void drawHorizLine(int row, int col, int length, u16 color) {

    int colfinal, i;

    if (length > 0) {
        colfinal = col + length;
        for (i = col; i < colfinal; i++) {
            setPixel(row, i, color);
        }
    } else if (length < 0) {
        colfinal = col + length;
        for (i = colfinal; i < col; i++) {
            setPixel(row, i, color);
        }
    }

}

void drawVertLine(int row, int col, int length, u16 color) {

    int rowfinal, i;
    if (length > 0) {
        rowfinal = row + length;
        for (i = row; i < rowfinal; i++) {
            setPixel(i, col, color);
        }
    } else if (length < 0) {
        rowfinal = row + length;
        for (i = rowfinal; i < row; i++) {
            setPixel(i, col, color);
        }
    }

}

void drawHollowRect(int row, int col, int height, int width, int thickness, u16 color) {
    int r = row;
    for (r = r; r < row + thickness; r++) {
        drawHorizLine(r, col, width, color);
    }
    int r1 = row + height - thickness;
    for (r1 = r1; r1 < row + height; r1++) {
        drawHorizLine(r1, col, width, color);
    }
    int c = col;
    c = c;
    for (c = c; c < col + thickness; c++) {
        drawVertLine(row + thickness, c, height - (thickness * 2), color);
    }
    int c1 = col + width - thickness;
    for (c1 = c1; c1 < col + width; c1++) {
        drawVertLine(row + thickness, c1, height - (thickness * 2), color);
    }
}

void rasterCircle(int row, int col, int radius, u16 color) {

    int f = 1 - radius;
    int ddF_x = 1;
    int ddF_y = -2 * radius;
    int x = 0;
    int y = radius;

    setPixel(row, col + radius, color);
    setPixel(row, col - radius, color);
    setPixel(row + radius, col, color);
    setPixel(row - radius, col, color);


    while (x < y) {
        if (f >= 0) {
            y--;
            ddF_y += 2;
            f += ddF_y;
        }
        x++;
        ddF_x += 2;
        f += ddF_x;
        setPixel(row + x, col + y, color);
        setPixel(row - x, col + y, color);
        setPixel(row + x, col - y, color);
        setPixel(row - x, col - y, color);
        setPixel(row + y, col + x, color);
        setPixel(row - y, col + x, color);
        setPixel(row + y, col - x, color);
        setPixel(row - y, col - x, color);
    }

}

void rasterCircleFace(int row, int col, int radius, u16 color) {

    int f = 1 - radius;
    int ddF_x = 1;
    int ddF_y = -2 * radius;
    int x = 0;
    int y = radius;

    setPixel(row, col + radius, color);
    setPixel(row, col - radius, color);
    setPixel(row + radius, col, color);
    setPixel(row - radius, col, color);

    while (x < y) {
        if (f >= 0) {
            y--;
            ddF_y += 2;
            f += ddF_y;
        }
        x++;
        ddF_x += 2;
        f += ddF_x;
        setPixel(row + x, col + y, color);
        setPixel(row - x, col + y, color);
        setPixel(row + x, col - y, color);
        setPixel(row - x, col - y, color);
        setPixel(row + y, col + x, color);
        setPixel(row - y, col + x, color);
        setPixel(row + y, col - x, color);
        setPixel(row - y, col - x, color);
        setPixel(row - (y / 2), col - (y / 2), color);
        setPixel(row - (y / 2), col + (y / 2), color);
        drawVertLine(row - (radius / 5), col, (radius / 3), color);
        drawHollowRect(row + (y / 2), col - (y / 5), (radius / 3), (radius / 3), 1, color);

    }
}

void grid(){

    drawRect(78, 0, 4, 75, WHITE);
    drawRect(0, 112, 122, 4, WHITE);
    drawRect(118, 117, 4, 90, WHITE);
    drawRect(78, 152, 4, 87, WHITE);

}

u16 changeColor() {

    int random = rand()*32;

    return COLOR(random, random, random);

}

void waitForVblank() {
    while (SCANLINECOUNTER > 160);
    while (SCANLINECOUNTER < 160);
}

void pause() {
    //volatile int x=0;
    volatile int delay;
     for (delay = 0; delay < 2000; delay++) {
       //  x++;//waste time
     }
}